/*
  filedialog.h
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "filedialog.h"

#include <stdlib.h>

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qhbox.h>
#include <qsizepolicy.h>
#include <qlineedit.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qmessagebox.h>

#define PICS_PATH "/opt/Qtopia/pics144/"

#define IMG_UP PICS_PATH"folderup.png"
#define IMG_NEW PICS_PATH"foldernew.png"
#define IMG_MAIN PICS_PATH"MainDeviceS.png"
#define IMG_SD PICS_PATH"SDDeviceS.png"
#define IMG_CF PICS_PATH"CFDeviceS.png"

#define IMG_FILE PICS_PATH"DocsIcon.png"
#define IMG_DIR PICS_PATH"slfolder.png"

#ifdef QWS
#define HOME_PATH "/home/zaurus/Documents"
#else
#define HOME_PATH "/home/tomo"
#endif
#define CF_PATH "/mnt/cf"
#define CARD_PATH "/mnt/card"

FileDialog::FileDialog(QWidget* parent, QString const &caption,
					   enum Type type, QString const &path,
					   bool selectable_dir)
	: QDialog( parent, "filedialog", TRUE, 0 )
{
	QSizePolicy minimum(QSizePolicy::Minimum, QSizePolicy::Minimum);
	QSizePolicy h_expand(QSizePolicy::Expanding, QSizePolicy::Minimum);

    resize(400, 400);

	setCaption(caption);

#ifdef QWS
	setFont(QFont("lcfont", 16));
#else
	setFont(QFont("gothic", 12));
#endif

	dir_select = selectable_dir;
	dialog_type = type;

	base_vbox = new QVBoxLayout(this);

	/* path box */
	path_hbox = new QHBox(this);
	path_hbox->setSpacing(3);
	path_combo = new QComboBox(path_hbox);
	path_combo->setSizePolicy(h_expand);
	up_button = new QPushButton(path_hbox);
	up_button->setPixmap(QPixmap(IMG_UP));
	up_button->setSizePolicy(minimum);
#ifndef FILEDIALOG_NO_NEW
	new_button = new QPushButton(path_hbox);
	new_button->setPixmap(QPixmap(IMG_NEW));
	new_button->setSizePolicy(minimum);
#endif

	base_vbox->addWidget(path_hbox);
	base_vbox->addSpacing(3);

	/* listbox */
	filelist = new QListBox(this);
	base_vbox->addWidget(filelist);
	base_vbox->addSpacing(3);

	/* filename */
	filename = new QLineEdit(this);
	filename->setSizePolicy(h_expand);
	base_vbox->addWidget(filename);
	base_vbox->addSpacing(3);

	/* button box */
	button_hbox = new QHBox(this);
	button_hbox->setSpacing(3);
	cancel_button = new QPushButton("Cancel", button_hbox);
	switch (dialog_type) {
	case Open:
		open_button = new QPushButton("Open", button_hbox);
		break;
	case Save:
		open_button = new QPushButton("Save", button_hbox);
		break;
	}
	base_vbox->addWidget(button_hbox);

	connect(open_button, SIGNAL(clicked()),
			this, SLOT(accept()));
	connect(cancel_button, SIGNAL(clicked()),
			this, SLOT(reject()));

	/* connect signal/slot */
	connect(path_combo, SIGNAL(activated(int)),
			this, SLOT(pathChanged(int)));
	connect(up_button, SIGNAL(clicked()),
			this, SLOT(upPressed()));
#ifndef FILEDIALOG_NO_NEW
	connect(new_button, SIGNAL(clicked()),
			this, SLOT(newPressed()));
#endif

	connect(filelist, SIGNAL(currentChanged(QListBoxItem *)),
			this, SLOT(listSelected(QListBoxItem *)));
	connect(filelist, SIGNAL(doubleClicked(QListBoxItem *)),
			this, SLOT(listDblClicked(QListBoxItem *)));

	/* init combobox */
	path_combo->insertItem(QPixmap(IMG_MAIN), QString::fromUtf8("本体メモリ"));
	path_list.append(HOME_PATH);

	if(!system("mount | grep -q card")){
		path_combo->insertItem(QPixmap(IMG_SD), QString::fromUtf8("SDカード"));
		path_list.append(CARD_PATH);
	}
	if(!system("mount | grep -q cf")){
		path_combo->insertItem(QPixmap(IMG_CF), QString::fromUtf8("CompactFlash"));
		path_list.append(CF_PATH);
	}

	if(path == ""){
		path_combo->setCurrentItem(0);
		pathChanged(0);
	}else{
		setFileName(path);
	}
}

FileDialog::~FileDialog()
{
}

QString FileDialog::getFileName()
{
	QString path = current_path + "/" + filename->text();
	QFileInfo finfo(path);

	if(finfo.isDir())
		return path + "/";

	return path;
}

void FileDialog::pathChanged(int index)
{
	QPixmap img_folder(IMG_DIR);
	QPixmap img_file(IMG_FILE);

	filelist->clear();

    QDir d(path_list[index]);
	d.setFilter(QDir::Dirs | QDir::Files);
	d.setSorting(QDir::Name | QDir::DirsFirst);

	if(!d.exists()){
		QMessageBox::critical(this, 
							  "Error!!",
							  path_list[index]
							  + QString::fromUtf8("が見つかりません"));
		return;
	}

	const QFileInfoList *list = d.entryInfoList();
	QFileInfoListIterator it( *list );
	QFileInfo *fi;

	while ( (fi=it.current()) ) {
		++it;
		if(fi->fileName() == "." || fi->fileName() == "..")
			continue;

		if(fi->isDir()){
			filelist->insertItem(img_folder, fi->fileName());
		}else{
			filelist->insertItem(img_file, fi->fileName());
		}
	}

	current_path = path_list[index];
}

void FileDialog::listSelected(QListBoxItem *item)
{
	QFileInfo file(current_path+"/"+item->text());

	if(!file.exists())
		return;
	if(!file.isDir() || dir_select){
		filename->setText(item->text());
	}
}

void FileDialog::cleanItem(int index)
{
	QString path = path_list[index];
	unsigned int i;

	i=0;
	while(i < path_list.count()){
		if(isRootPath(path_list[i]) ||
		   path.find(path_list[i]) == 0){
			i++;
			continue;
		}

		path_combo->removeItem(i);
		path_list.remove(path_list.at(i));
	}
}

void FileDialog::listDblClicked(QListBoxItem *item)
{
	QString new_path = current_path+"/"+item->text();
	QFileInfo file(new_path);

	if(!file.exists())
		return;

	if(!file.isDir()){
		/* if double clicked a file */
		accept();
		return;
	}

	/* dir */
	int path_index = path_combo->currentItem();

	cleanItem(path_index);

	path_index = path_combo->currentItem();

	path_combo->insertItem(QPixmap(IMG_DIR), item->text(), path_index + 1);
	path_list.insert(path_list.at(path_index + 1), new_path);

	path_combo->setCurrentItem(path_index + 1);
	pathChanged(path_index + 1);

	filename->setText("");
}

void FileDialog::upPressed()
{
	int path_index = path_combo->currentItem();
	if(isRootPath(path_list[path_index]))
		return;

	path_index--;
	path_combo->setCurrentItem(path_index);
	pathChanged(path_index);
}

#ifndef FILEDIALOG_NO_NEW
void FileDialog::newPressed()
{
}
#endif

void FileDialog::accept()
{
	QString path = current_path + "/" + filename->text();
	QFileInfo file(path);

	switch(dialog_type){
	case Open:
		if(!file.exists()){
			QMessageBox::critical(this, 
								  "Error!!",
								  filename->text()
								  + QString::fromUtf8(" が見つかりません"));
			return;
		}
		break;
	case Save:
		if(file.exists()){
			int ret = QMessageBox::information(this, 
											   "Overwrite?",
											   filename->text()
											   + QString::fromUtf8(" は存在します。\n"
																   "上書きしてもよろしいですか?")
											   , 2, 1);
			if(ret != 1)
				return;
		}
		break;
	}
	QDialog::accept();
}

bool FileDialog::isRootPath(QString str)
{
	return str == HOME_PATH || str == CF_PATH || str == CARD_PATH;
}

void FileDialog::setFileName(QString const &file)
{
	unsigned int i, index;
	QString path;

	for(index = 0; index < path_list.count(); index++){
		if(file.find(path_list[index]) == 0)
			break;
	}

	if(index >= path_list.count())
		return;

	path = path_list[index];
	QStringList sl;
	sl = QStringList::split("/", file.mid(path.length()+1));

	for(i=0; i < sl.count()-1; i++){
		path += "/" + sl[i];

		QFileInfo fi(path);
		if(!fi.exists() || !fi.isDir())
			break;

		path_combo->insertItem(QPixmap(IMG_DIR), sl[i], index + i + 1);
		path_list.insert(path_list.at(index + i + 1), path);
	}

	path_combo->setCurrentItem(index + i);
	pathChanged(index + i);

	QListBoxItem *item = filelist->findItem(sl[sl.count()-1]);
	if(item)
		filelist->setCurrentItem(item);
}
