/*
  filedialog.h
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __FILEDIALOG_H__
#define __FILEDIALOG_H__

#include <qvariant.h>
#include <qdialog.h>
#include <qstringlist.h>

class QComboBox;
class QPushButton;
class QHGroupBox;
class QVBoxLayout;
class QHBox;
class QListBox;
class QListBoxItem;
class QString;
class QLineEdit;

#define FILEDIALOG_NO_NEW

class FileDialog : public QDialog
{ 
    Q_OBJECT

  public:
	enum Type {
		Open,
		Save,
	};

    FileDialog(QWidget* parent = 0, QString const &caption = "",
			   enum Type type = Open, QString const &path = "",
			   bool selectable_dir = false);
    ~FileDialog();

	QString getFileName();
	void setFileName(QString const &file);
	void accept();

  protected:

	Type dialog_type;

	QVBoxLayout *base_vbox;
	QHBox *path_hbox;
	QHBox *button_hbox;

	QComboBox *path_combo;
	QPushButton *up_button;
#ifndef FILEDIALOG_NO_NEW
	QPushButton *new_button;
#endif
	QListBox *filelist;

	QLineEdit *filename;
	QPushButton *open_button;
	QPushButton *cancel_button;

	QStringList path_list;
	QString current_path;

	void cleanItem(int index);

	bool isRootPath(QString str);

	bool dir_select;

  public slots:
	void pathChanged(int index);
	void listSelected(QListBoxItem *item);
	void listDblClicked(QListBoxItem *item);
	void upPressed();
#ifndef FILEDIALOG_NO_NEW
//	void newPressed();
#endif
};

#endif // __FILEDIALOG_H__
