/*
  gpsinfo.h
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GPSINFO_H__
#define __GPSINFO_H__

#include <pthread.h>
#include <qstring.h>
#include <qfile.h>
#include <qdatetime.h>

class Pos;

class GPSInfo {
  public:
	GPSInfo(void);
	~GPSInfo();

	void updateInfo(QString const &nmea);
	GPSInfo copy();

	QDateTime getDateTime();
	double getLatitude();
	double getLongitude();
	double getAltitude();
	int getSatellites();
	double getVelocity();
	int getDegree();
	bool isUpdated();
	bool isValid();

	void convTokyoToWGS();
	void setTimeZone(int timezone);

  protected:
	pthread_mutex_t lock;

	bool is_updated;
	bool is_valid;

	QDateTime datetime;
	double latitude;
	double longitude;
	double altitude;
	int satellites;
	int status;

	double velocity;
	int degree;

	QTime decodeNMEATime(QString const &time);
	double decodeNMEALatitude(QString const &lat_str,
							  QString const &dir);
	double decodeNMEALongitude(QString const &lon_str,
							   QString const &dir);

};

#endif /* __GPSINFO_H__ */
