/*
  gpsmenu.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>

#include <qpainter.h>
#include <qimage.h>
#include <qbitmap.h>
#include <qcursor.h>

#include "qgmap.h"
#include "mainwindow.h"
#include "gpsmenu.h"
#include "dirdialog.h"
#include "gammadialog.h"
#include "gpsdialog.h"

#include "mappathdialog.h"

GPSMenu::GPSMenu(QWidget *parent, const char *name) :
	QPopupMenu(parent, name)
{
	insertItem("Settings", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTINGS);
	insertSeparator();
	insertItem("GPS1", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTING1);
	insertItem("GPS2", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTING2);
	insertItem("GPS3", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTING3);
	insertItem("GPS4", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTING4);
	insertItem("GPS5", this, SLOT(gps_menu_activated(int)),0, GPS_SUB_SETTING5);

	loadConfig();
}

void GPSMenu::gps_menu_activated(int id)
{
	int ret;

	if(id == GPS_SUB_SETTINGS){
		GpsDialog dlg(parentWidget(), "GPS", TRUE);

		dlg.loadConfig();
		ret = dlg.exec();
		if(ret == QDialog::Accepted){
			printf("accepted\n");
			dlg.saveConfig();
			loadConfig();
		}
		return;
	}

	setActive(id);

	QGConf *c = (QGConf *)lookupObject(this, "QGConf", "config");
	if(!c)
		return;

	c->writeEntry("GPSSelect", id);
}

void GPSMenu::loadConfig()
{
	QGConf *c = (QGConf *)lookupObject(this, "QGConf", "config");
	if(!c)
		return;

	int i;
	for(i = GPS_SUB_SETTING1; i <= GPS_SUB_SETTING5; i++){
		QString group;
		group.sprintf("GPSDevice%d", i);
		c->setGroup(group);

		changeItem(i, c->readEntry("Name", group));
	}
	c->setGroup("");

	setActive(c->readNumEntry("GPSSelect", 1));
}

void GPSMenu::setActive(int id)
{
	int i;
	for(i = GPS_SUB_SETTING1; i <= GPS_SUB_SETTING5; i++)
		setItemChecked(i, i == id);
}

void GPSMenu::setEnable(bool enable)
{
	int i;
	for(i = GPS_SUB_SETTINGS; i <= GPS_SUB_SETTING5; i++)
		setItemEnabled(i, enable);
}
