/*
  iconmenu.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>

#include <qpainter.h>
#include <qimage.h>
#include <qbitmap.h>
#include <qcursor.h>

#include "qgmap.h"
#include "mainwindow.h"
#include "iconmenu.h"
#include "dirdialog.h"
#include "gammadialog.h"
#include "gpsdialog.h"

#include "mappathdialog.h"
#include "gpsmenu.h"

IconMenu::IconMenu(LayeredBase *parent, const char *name) :
	LayeredWidget(parent, name),
	gps_sub_menu(parent, "gps_sub_menu")
{
	icon.load(MENUIMG_FILE);
	setGeometry(10,10, icon.width(), icon.height());

	popup_menu.insertItem("Map Path", this, SLOT(menu_activated(int)),0,MENU_MAPPATH);
	popup_menu.insertItem("Gamma", this, SLOT(menu_activated(int)),0,MENU_GAMMA);
	popup_menu.insertItem("GPS", &gps_sub_menu);

	connect(this, SIGNAL(reload(bool)),
			parent, SIGNAL(reload(bool)));
	connect(this, SIGNAL(cacheFlush()),
			parent, SIGNAL(cacheFlush()));
}

bool IconMenu::paintEvent(QPaintEvent *e UNUSED, QPixmap &scr)
{
	QPainter p(&scr);
	p.drawPixmap(10, 10, icon);
	return true;
}

bool IconMenu::mousePressEvent( QMouseEvent *e )
{
	if(e->button() == LeftButton)
		popup_menu.popup(QCursor::pos());

	return false;
}

void IconMenu::menu_activated(int id)
{
	int ret;
	QGConf *conf = (QGConf *)lookupObject(this, "QGConf", "config");

	switch(id){
	case MENU_MAPPATH:
	{
		MapPathDialog dlg(parentWidget(), QString::fromUtf8("Select 'imgs' path in GM_Lite"));
		dlg.loadConfig();
		ret = dlg.exec();
		if(ret == QDialog::Accepted){
			dlg.saveConfig();
			emit cacheFlush();
			emit reload(true);
		}
		break;
	}
	case MENU_GAMMA:
	{
		GammaDialog dlg(parentWidget(), "Gamma", TRUE);

		QString gamma = conf->readEntry("Gamma", DEFAULT_GAMMA);
		dlg.setDefault(gamma.toDouble());

		ret = dlg.exec();
		if(ret == QDialog::Accepted){
			conf->writeEntry("Gamma", QString::number(dlg.getGamma()));
		}
		break;
	}
	}
}

bool IconMenu::keyPressEvent(QKeyEvent *e)
{
	if(e->key() == Key_F11){
		QPoint pos = parentWidget()->geometry().topLeft();
		pos += QPoint(10 + icon.width()/2, 10 + icon.height()/2);

		popup_menu.popup(pos);
		return true;
	}
	return false;
}

void IconMenu::request(QString const &req, QString const &arg)
{
	if(req == "disable" && arg == "GPSMenu"){
		gps_sub_menu.setEnable(false);
	}else if(req == "enable" && arg == "GPSMenu"){
		gps_sub_menu.setEnable(true);
	}
}
